BVP

Struct BVP 

pub struct BVP<EqType, T: Real, Y: State<T>, Method, SoloutType> { /* private fields */ }
Builder for boundary value problems.

Use BVP::ode for an ODE system that implements ODE and Boundary, or BVP::ode_from_fn when closures are more convenient.

Implementations

Source

impl<'a, F, T, Y> BVP<&'a F, T, Y, (), DefaultSolout>
where T: Real, Y: State<T>, F: ODE<T, Y> + Boundary<T, Y> + ?Sized,

pub fn ode(system: &'a F, t0: T, tf: T, y_guess: Y) -> Self

Create a boundary value problem for an ODE.

Source

impl<F, B, T, Y> BVP<OdeBvpFromFn<F, B>, T, Y, (), DefaultSolout>
where T: Real, Y: State<T>, F: Fn(T, &Y, &mut Y), B: Fn(&Y, &Y, &mut Y),

pub fn ode_from_fn(diff: F, boundary: B, t0: T, tf: T, y_guess: Y) -> Self

Create a boundary value problem for an ODE from closures.

Source

impl<EqType, T: Real, Y: State<T>, Method, SoloutType> BVP<EqType, T, Y, Method, SoloutType>

pub fn method<NextMethod>( self, method: NextMethod, ) -> BVP<EqType, T, Y, NextMethod, SoloutType>

Set the numerical method used to solve the BVP.

pub fn solout<NextSolout>( self, solout: NextSolout, ) -> BVP<EqType, T, Y, Method, NextSolout>

Set a custom solout function for the final converged trajectory.

pub fn even(self, dt: T) -> BVP<EqType, T, Y, Method, EvenSolout<T>>

Output evenly spaced points between the initial and final time.

This controls the output of the final converged IVP trajectory.

pub fn dense(self, n: usize) -> BVP<EqType, T, Y, Method, DenseSolout>

Output dense interpolation points between accepted solver steps.

This controls the output of the final converged IVP trajectory.

pub fn t_eval( self, points: impl AsRef<[T]>, ) -> BVP<EqType, T, Y, Method, TEvalSolout<T>>

Use provided time points for final trajectory output.

pub fn event<'a, E>( self, event: &'a E, ) -> BVP<EqType, T, Y, Method, EventWrappedSolout<'a, T, Y, SoloutType, E>>
where E: Event<T, Y>, SoloutType: Solout<T, Y>,

Wrap current final-trajectory output with event detection.

pub fn crossing( self, component_idx: usize, threshold: T, direction: CrossingDirection, ) -> BVP<EqType, T, Y, Method, CrossingSolout<T>>

Output points where a component crosses a threshold on the final trajectory.

pub fn hyperplane_crossing<Y1: State<T>>( self, point: Y1, normal: Y1, extractor: fn(&Y) -> Y1, direction: CrossingDirection, ) -> BVP<EqType, T, Y, Method, HyperplaneCrossingSolout<T, Y1, Y>>

Output points where a projected final trajectory crosses a hyperplane.

Source

impl<EqType, T, Y, Method, SoloutType> BVP<EqType, T, Y, Method, SoloutType>
where T: Real, Y: State<T>, EqType: ODE<T, Y> + Boundary<T, Y>, Method: BVPMethod<T, Y>, SoloutType: Solout<T, Y>,

pub fn solve(self) -> Result<Solution<T, Y>, Error<T, Y>>

Solve the boundary value problem.