pub struct BVP<EqType, T: Real, Y: State<T>, Method, SoloutType> { /* private fields */ }Expand description
Builder for boundary value problems.
Use BVP::ode for an ODE system that implements ODE and Boundary,
or BVP::ode_from_fn when closures are more convenient.
Implementations
Sourceimpl<F, B, T, Y> BVP<OdeBvpFromFn<F, B>, T, Y, (), DefaultSolout>
impl<F, B, T, Y> BVP<OdeBvpFromFn<F, B>, T, Y, (), DefaultSolout>
Sourcepub fn ode_from_fn(diff: F, boundary: B, t0: T, tf: T, y_guess: Y) -> Self
pub fn ode_from_fn(diff: F, boundary: B, t0: T, tf: T, y_guess: Y) -> Self
Create a boundary value problem for an ODE from closures.
Sourceimpl<EqType, T: Real, Y: State<T>, Method, SoloutType> BVP<EqType, T, Y, Method, SoloutType>
impl<EqType, T: Real, Y: State<T>, Method, SoloutType> BVP<EqType, T, Y, Method, SoloutType>
Sourcepub fn method<NextMethod>(
self,
method: NextMethod,
) -> BVP<EqType, T, Y, NextMethod, SoloutType>
pub fn method<NextMethod>( self, method: NextMethod, ) -> BVP<EqType, T, Y, NextMethod, SoloutType>
Set the numerical method used to solve the BVP.
Sourcepub fn solout<NextSolout>(
self,
solout: NextSolout,
) -> BVP<EqType, T, Y, Method, NextSolout>
pub fn solout<NextSolout>( self, solout: NextSolout, ) -> BVP<EqType, T, Y, Method, NextSolout>
Set a custom solout function for the final converged trajectory.
Sourcepub fn even(self, dt: T) -> BVP<EqType, T, Y, Method, EvenSolout<T>>
pub fn even(self, dt: T) -> BVP<EqType, T, Y, Method, EvenSolout<T>>
Output evenly spaced points between the initial and final time.
This controls the output of the final converged IVP trajectory.
Sourcepub fn dense(self, n: usize) -> BVP<EqType, T, Y, Method, DenseSolout>
pub fn dense(self, n: usize) -> BVP<EqType, T, Y, Method, DenseSolout>
Output dense interpolation points between accepted solver steps.
This controls the output of the final converged IVP trajectory.
Sourcepub fn t_eval(
self,
points: impl AsRef<[T]>,
) -> BVP<EqType, T, Y, Method, TEvalSolout<T>>
pub fn t_eval( self, points: impl AsRef<[T]>, ) -> BVP<EqType, T, Y, Method, TEvalSolout<T>>
Use provided time points for final trajectory output.
Sourcepub fn event<'a, E>(
self,
event: &'a E,
) -> BVP<EqType, T, Y, Method, EventWrappedSolout<'a, T, Y, SoloutType, E>>
pub fn event<'a, E>( self, event: &'a E, ) -> BVP<EqType, T, Y, Method, EventWrappedSolout<'a, T, Y, SoloutType, E>>
Wrap current final-trajectory output with event detection.
Sourcepub fn crossing(
self,
component_idx: usize,
threshold: T,
direction: CrossingDirection,
) -> BVP<EqType, T, Y, Method, CrossingSolout<T>>
pub fn crossing( self, component_idx: usize, threshold: T, direction: CrossingDirection, ) -> BVP<EqType, T, Y, Method, CrossingSolout<T>>
Output points where a component crosses a threshold on the final trajectory.
Sourcepub fn hyperplane_crossing<Y1: State<T>>(
self,
point: Y1,
normal: Y1,
extractor: fn(&Y) -> Y1,
direction: CrossingDirection,
) -> BVP<EqType, T, Y, Method, HyperplaneCrossingSolout<T, Y1, Y>>
pub fn hyperplane_crossing<Y1: State<T>>( self, point: Y1, normal: Y1, extractor: fn(&Y) -> Y1, direction: CrossingDirection, ) -> BVP<EqType, T, Y, Method, HyperplaneCrossingSolout<T, Y1, Y>>
Output points where a projected final trajectory crosses a hyperplane.