pub struct BackwardDifferentiationFormula<E, T: Real, Y: State<T>> {}Expand description
Adaptive variable-order Backward Differentiation Formula solver.
Uses Backward Differentiation Formula orders 1-5 with automatic order and step size selection. Newton iteration resolves the implicit Backward Differentiation Formula equations. Dense output via polynomial interpolation is supported.
The solver starts at order 1 and considers increasing the order after enough successful steps. On step rejection, the order is decreased.
Fields
h0: Trtol: Tolerance<T>atol: Tolerance<T>h_max: Th_min: Tmax_steps: usizesafety_factor: Tmin_scale: Tmax_scale: Tfilter: fn(T) -> Tnewton_tol: Tmax_newton_iter: usizemax_order: usizeImplementations
Sourceimpl<E, T: Real, Y: State<T>> BackwardDifferentiationFormula<E, T, Y>
impl<E, T: Real, Y: State<T>> BackwardDifferentiationFormula<E, T, Y>
pub fn adaptive() -> Self
pub fn rtol<V: Into<Tolerance<T>>>(self, rtol: V) -> Self
pub fn atol<V: Into<Tolerance<T>>>(self, atol: V) -> Self
pub fn h0(self, h0: T) -> Self
pub fn h_min(self, h_min: T) -> Self
pub fn h_max(self, h_max: T) -> Self
pub fn max_steps(self, max_steps: usize) -> Self
pub fn safety_factor(self, safety_factor: T) -> Self
pub fn newton_tol(self, newton_tol: T) -> Self
pub fn max_newton_iter(self, max_newton_iter: usize) -> Self
pub fn max_order(self, max_order: usize) -> Self
pub fn filter(self, filter: fn(T) -> T) -> Self
pub fn order(&self) -> usize